@@ -3,7 +3,7 @@ import pandas as pd
 from scipy import spatial
 from SpatialCluster.utils.data_format import data_format, position_data_format
 
-def rings(features_X, features_position, max_radio=[0.00014, 0.00024, 0.00034], max_neighbors_per_radio=[200, 500, 1000], keep_original_value=True, leafsize=10):
+def rings(features_X, features_position, max_radio=[0.00014, 0.00024, 0.00034], max_neighbors_per_radio=[200, 500, 1000], keep_original_value=True, smoothing=1e-08, normalize=True, leafsize=10):
 
     """
     El algoritmo toma todos los puntos dentro de un radio máximo.
@@ -16,7 +16,7 @@ def rings(features_X, features_position, max_radio=[0.00014, 0.00024, 0.00034],
     assert len(max_radio) == len(max_neighbors_per_radio)
 
     pts = np.array(list(zip(features_position.lon, features_position.lat)))
-    tree = spatial.KDTree(data=pts, leafsize=leafsize)
+    tree = spatial.cKDTree(data=pts, leafsize=leafsize)
     final_features_df = pd.DataFrame()
     
     if keep_original_value:
@@ -30,20 +30,20 @@ def rings(features_X, features_position, max_radio=[0.00014, 0.00024, 0.00034],
 
         for point in range(len(ball_points)):               # Para cada punto
             idxs = np.array(ball_points[point])                 # Solo consigo puntos con cierto radio
-            
-            if len(idxs) == 1: 
-                nearby_points = [0]
-            else:
-                temporal_tree = spatial.KDTree(data=pts[idxs], leafsize=leafsize)
-                k_length = min(k, len(idxs))            # Conseguir los K puntos más cercanos dentro de ese radio
-                nearby_points = temporal_tree.query(pts[point], k = k_length)[1] 
-            
-            indexs = idxs[nearby_points]
-
-            for f_index, feature in enumerate(features_X.columns): 
-                subset_feature = features_X.loc[indexs, :][feature] 
+            temporal_tree = spatial.cKDTree(data=pts[idxs], leafsize=leafsize)
+            k_length = min(k, len(idxs))            # Conseguir los K puntos más cercanos dentro de ese radio
+            distances, nearby_points = temporal_tree.query(pts[point], k = k_length)
+            distances += smoothing
+            if(normalize):
+                distances = distances/np.linalg.norm(distances)
+            try:
+                indexs = idxs[nearby_points]
+            except:
+                print("Error nearby points")
+            for f_index, feature in enumerate(features_X.columns):
+                subset_feature = features_X[feature].values/distances
                 mean_feature = subset_feature.mean()
-                features_array[f_index].append(mean_feature)        
+                features_array[f_index].append(mean_feature) 
         
         for f_index, feature in enumerate(features_X.columns):
             final_features_df[f"{feature}_{i}"] = features_array[f_index]
